//
// Created by #Suyghur, on 2021/10/15.
//


#ifndef XINRUIGAMESDK_AES_H
#define XINRUIGAMESDK_AES_H

#include <cstdint>

// #define the macros below to 1/0 to enable/disable the mode of operation.
//
// CBC enables AES encryption in CBC-mode of operation.
// CTR enables encryption in counter-mode.
// ECB enables the basic ECB 16-byte block algorithm. All can be enabled simultaneously.

// The #ifndef-guard allows it to be configured before #include'ing or at compile time.
#ifndef CBC
#define CBC 1
#endif

#ifndef ECB
#define ECB 1
#endif

#ifndef CTR
#define CTR 1
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define AES128 1

#if defined(ECB) && (ECB == 1)

void AesEcbEncrypt(const uint8_t *input, const uint8_t *key, uint8_t *output, const uint32_t length);

void AesEcbDecrypt(const uint8_t *input, const uint8_t *key, uint8_t *output, const uint32_t length);

#endif // #if defined(ECB) && (ECB == 1)

#if defined(CBC) && (CBC == 1)

void AesCbcEncryptBuffer(uint8_t *output, uint8_t *input, uint32_t length, const uint8_t *key, const uint8_t *iv);

void AesCbcDecryptBuffer(uint8_t *output, uint8_t *input, uint32_t length, const uint8_t *key, const uint8_t *iv);

#endif // #if defined(CBC)&&(CBC==1)

#if defined(CTR) && (CTR == 1)

/* Same function for encrypting as for decrypting. Note no IV/nonce should ever be reused with the same key */
void AesCtrXcryptBuffer(uint8_t *output, uint8_t *input, uint32_t length, const uint8_t *key, const uint8_t *nonce);

#endif // #if defined(CTR) && (CTR == 1)

#ifdef __cplusplus
}
#endif


#endif //XINRUIGAMESDK_AES_H
